
#include <kgi/module.h>
#include <kgi/system.h>
#include <3dfx/banshee.h>

#define ATCI  (IOBASE+0xC0)
#define ATCDr (IOBASE+0xC1)
#define ATCDw (IOBASE+0xC0)
#define MISCw (IOBASE+0xC2)
#define SEQI  (IOBASE+0xC4)
#define SEQD  (IOBASE+0xC5)
#define MISCr (IOBASE+0xCC)
#define GRCI  (IOBASE+0xCE)
#define GRCD  (IOBASE+0xCF)
#define CRTI  (IOBASE+0xD4)
#define CRTD  (IOBASE+0xD5)
#define INS1r (IOBASE+0xDA)

void 
save_state(struct kgi_display *dpy,struct statetable *state)
{
  kgiu8 i;
  GET_PRIVATE;
  state->MISC  = io_in8(MISCr);	/* save miscellaneous output reg */

  /* save sequencer regs */
  for(i = 0; i < NoSEQRegs; i++) {
    io_out8(i, SEQI); 
    state->SEQ[i]=io_in8(SEQD); 
  }

  /* save cathode ray tube controller regs */
  for(i = 0; i < NoCRTRegs; i++) {
    io_out8(i, CRTI);
    state->CRT[i]=io_in8(CRTD); 
  } 

  /* save graphics controller regs */
  for(i = 0; i < NoGRCRegs; i++) {
    io_out8(i, GRCI); 
    state->GRC[i]=io_in8(GRCD); 
  }

  /* save attribute controller regs */
  for(i = 0; i <NoATCRegs ; i++) {
    io_in8(INS1r);  /* reset AFF and setup index */
    io_out8(i, ATCI); 
    state->ATC[i]=io_in8(ATCDr); 
  }

  state->vgainit1=readl(IOMEMBASE+VGAINIT1);
  state->vidcfg=readl(IOMEMBASE+VIDPROCCFG);
  state->vidpll=readl(IOMEMBASE+PLLCTRL0 );
  state->mempll=readl(IOMEMBASE+PLLCTRL1); 
  state->gfxpll=readl(IOMEMBASE+PLLCTRL2);

  io_out8(0x1A,CRTI);
  state->ExtVga[0]= io_in8(CRTD);
  io_out8(0x1B,CRTI);
  state->ExtVga[1]= io_in8(CRTD);

  state->vgainit0=readl(IOMEMBASE+VGAINIT0);
  state->dacmode=readl(IOMEMBASE+DACMODE);
  state->stride=readl(IOMEMBASE+VIDDESKTOPOVERLAYSTRIDE);
  state->startaddr=readl(IOMEMBASE+VIDDESKTOPSTARTADDR);
  state->cursloc=readl(IOMEMBASE+HWCURPATADDR);
  state->screensize=readl(IOMEMBASE+VIDSCREENSIZE);
  state->vidcfg=readl(IOMEMBASE+VIDPROCCFG);
  state->vgainit1=readl(IOMEMBASE+VGAINIT1);
} 

void  
restore_state(struct kgi_display *dpy,struct statetable *state)
{

  int miscInit1;kgiu8 i; 
  GET_PRIVATE;

  /* restore_state is called as a mode set after the clock is
   * set in kgim_clock_set.  Do not set the clock here!! */
  
  miscInit1 = readl(IOMEMBASE+MISCINIT1);
  /*
    if ( memType == MEM_TYPE_SDRAM ) {
    miscInit1 |= SST_DISABLE_2D_BLOCK_WRITE;
    }
    */
  miscInit1|=1;
  writel(miscInit1,IOMEMBASE+MISCINIT1);
  
  /* unlock timing */
  io_out8(0x11, CRTI);
  io_out8(io_in8(CRTD) & 0x7f ,CRTD) ;

  writel(state->vgainit1 & 0x1FFFFF,IOMEMBASE+VGAINIT1);
  writel(state->vidcfg & (~0x1) ,IOMEMBASE+VIDPROCCFG);
  writel(state->mempll,IOMEMBASE+PLLCTRL1); 
  writel(state->gfxpll,IOMEMBASE+PLLCTRL2);
  writel(state->vidpll ,IOMEMBASE+PLLCTRL0);

  io_out8((io_in8(MISCr) & 0x0C) | state->MISC | 0x01, MISCw);

  for(i=0; i < NoSEQRegs ; i++) 
    outw((state->SEQ[i] << 8) | i, SEQI); 

  state->CRT[0x03] |= 0x80;
  state->CRT[0x11] &= ~0x80;
  for(i = 0; i < NoCRTRegs; i++) 
    outw((state->CRT[i] << 8) | i, CRTI); 

  for(i = 0; i < NoGRCRegs; i++) 
    outw((state->GRC[i] << 8 ) | i, GRCI); 

  inb(INS1r) ; 
  for (i = 0; i < NoATCRegs; i++) {
    io_out8(i, ATCI); 
    io_out8(state->ATC[i], ATCDw);
  } 

  outw((state->ExtVga[0]<<8) | 0x1A, CRTI);
  outw((state->ExtVga[1]<<8) | 0x1B, CRTI);

  writel(state->vgainit0,IOMEMBASE+VGAINIT0);
  writel(state->dacmode,IOMEMBASE+DACMODE);
  writel(state->stride,IOMEMBASE+VIDDESKTOPOVERLAYSTRIDE);
  writel(state->startaddr,IOMEMBASE+VIDDESKTOPSTARTADDR);
  /*  writel(IOMEMBASE+HWCURPATADDR, state->cursloc);*/
  writel(state->screensize,IOMEMBASE+VIDSCREENSIZE);
  writel(state->vidcfg,IOMEMBASE+VIDPROCCFG);
  writel(state->vgainit1,IOMEMBASE+VGAINIT1);

}

